import aioboto3
import logging
import qiniu

from typing import List, Dict, Any, Optional
from botocore.config import Config as S3Config

from ...config import config
from ...consts import consts

logger = logging.getLogger(consts.LOGGER_NAME)


class StorageService:
    def __init__(self, cfg: config.Config = None):
        # Configure boto3 with retries and timeouts
        self.s3_config = S3Config(
            retries=dict(max_attempts=2, mode="adaptive"),
            connect_timeout=30,
            read_timeout=60,
            max_pool_connections=50,
        )
        self.config = cfg
        self.s3_session = aioboto3.Session()
        self.auth = qiniu.Auth(cfg.access_key, cfg.secret_key)
        self.bucket_manager = qiniu.BucketManager(self.auth, preferred_scheme="https")

    def get_object_url(
            self, bucket: str, key: str, disable_ssl: bool = False, expires: int = 3600
    ) -> list[dict[str:Any]]:
        # 获取下载域名
        domains_getter = getattr(self.bucket_manager, "_BucketManager__uc_do_with_retrier")
        domains_list, domain_response = domains_getter('/v3/domains?tbl={0}'.format(bucket))
        if domain_response.status_code != 200:
            raise Exception(
                f"get bucket domain error：{domain_response.exception} reqId:{domain_response.req_id}"
            )

        if not domains_list or len(domains_list) == 0:
            raise Exception(
                f"get bucket domain error：domains_list is empty reqId:{domain_response.req_id}"
            )

        http_schema = "https" if not disable_ssl else "http"
        object_public_urls = []
        for domain in domains_list:
            # 被冻结
            freeze_types = domain.get("freeze_types")
            if freeze_types is not None:
                continue

            domain_url = domain.get("domain")
            if domain_url is None:
                continue

            object_public_urls.append({
                "object_url": f"{http_schema}://{domain_url}/{key}",
                "domain_type": "cdn" if domain.get("domaintype") is None or domain.get("domaintype") == 0 else "origin"
            })

        object_urls = []
        bucket_info, bucket_info_response = self.bucket_manager.bucket_info(bucket)
        if domain_response.status_code != 200:
            raise Exception(
                f"get bucket domain error：{bucket_info_response.exception} reqId:{bucket_info_response.req_id}"
            )
        if bucket_info["private"] != 0:
            for url_info in object_public_urls:
                public_url = url_info.get("object_url")
                if public_url is None:
                    continue
                url_info["object_url"] = self.auth.private_download_url(public_url, expires=expires)
                object_urls.append(url_info)
        else:
            for url_info in object_public_urls:
                object_urls.append(url_info)
        return object_urls

    async def list_buckets(self, prefix: Optional[str] = None) -> List[dict]:
        if not self.config.buckets or len(self.config.buckets) == 0:
            return []

        max_buckets = 50

        async with self.s3_session.client(
                "s3",
                aws_access_key_id=self.config.access_key,
                aws_secret_access_key=self.config.secret_key,
                endpoint_url=self.config.endpoint_url,
                region_name=self.config.region_name,
        ) as s3:
            # If buckets are configured, only return those
            response = await s3.list_buckets()
            all_buckets = response.get("Buckets", [])

            configured_bucket_list = [
                bucket
                for bucket in all_buckets
                if bucket["Name"] in self.config.buckets
            ]

            if prefix:
                configured_bucket_list = [
                    b for b in configured_bucket_list if b["Name"] > prefix
                ]

            return configured_bucket_list[:max_buckets]

    async def list_objects(
            self, bucket: str, prefix: str = "", max_keys: int = 20, start_after: str = ""
    ) -> List[dict]:
        if self.config.buckets and bucket not in self.config.buckets:
            logger.warning(f"Bucket {bucket} not in configured bucket list")
            return []

        if isinstance(max_keys, str):
            max_keys = int(max_keys)

        if max_keys > 100:
            max_keys = 100

        async with self.s3_session.client(
                "s3",
                aws_access_key_id=self.config.access_key,
                aws_secret_access_key=self.config.secret_key,
                endpoint_url=self.config.endpoint_url,
                region_name=self.config.region_name,
        ) as s3:
            response = await s3.list_objects_v2(
                Bucket=bucket,
                Prefix=prefix,
                MaxKeys=max_keys,
                StartAfter=start_after,
            )
            return response.get("Contents", [])

    async def get_object(self, bucket: str, key: str) -> Dict[str, Any]:
        if self.config.buckets and bucket not in self.config.buckets:
            logger.warning(f"Bucket {bucket} not in configured bucket list")
            return {}

        async with self.s3_session.client(
                "s3",
                aws_access_key_id=self.config.access_key,
                aws_secret_access_key=self.config.secret_key,
                endpoint_url=self.config.endpoint_url,
                region_name=self.config.region_name,
                config=self.s3_config,
        ) as s3:
            # Get the object and its stream
            response = await s3.get_object(Bucket=bucket, Key=key)
            stream = response["Body"]

            # Read the entire stream in chunks
            chunks = []
            async for chunk in stream:
                chunks.append(chunk)

            # Replace the stream with the complete data
            response["Body"] = b"".join(chunks)
            return response

    def upload_text_data(self, bucket: str, key: str, data: str, overwrite: bool = False) -> list[dict[str:Any]]:
        policy = {
            "insertOnly": 1,
        }

        if overwrite:
            policy["insertOnly"] = 0
            policy["scope"] = f"{bucket}:{key}"

        token = self.auth.upload_token(bucket=bucket, key=key, policy=policy)
        ret, info = qiniu.put_data(up_token=token, key=key, data=bytes(data, encoding="utf-8"))
        if info.status_code != 200:
            raise Exception(f"Failed to upload object: {info}")

        return self.get_object_url(bucket, key)

    def upload_local_file(self, bucket: str, key: str, file_path: str, overwrite: bool = False) -> list[dict[str:Any]]:
        policy = {
            "insertOnly": 1,
        }

        if overwrite:
            policy["insertOnly"] = 0
            policy["scope"] = f"{bucket}:{key}"

        token = self.auth.upload_token(bucket=bucket, key=key, policy=policy)
        ret, info = qiniu.put_file(up_token=token, key=key, file_path=file_path)
        if info.status_code != 200:
            raise Exception(f"Failed to upload object: {info}")

        return self.get_object_url(bucket, key)

    def fetch_object(self, bucket: str, key: str, url: str):
        ret, info = self.bucket_manager.fetch(url, bucket, key=key)
        if info.status_code != 200:
            raise Exception(f"Failed to fetch object: {info}")

        return self.get_object_url(bucket, key)

    def is_text_file(self, key: str) -> bool:
        text_extensions = {
            ".ini",
            ".conf",
            ".py",
            ".js",
            ".xml",
            ".yml",
            ".properties",
            ".txt",
            ".log",
            ".json",
            ".yaml",
            ".md",
            ".csv",
            ".html",
            ".css",
            ".sh",
            ".bash",
            ".cfg",
        }
        return any(key.lower().endswith(ext) for ext in text_extensions)


    def is_image_file(self, key: str) -> bool:
        """Determine if a file is text-based by its extension"""
        text_extensions = {
            ".gif",
            ".png",
            ".jpg",
            ".bmp",
            ".jpeg",
            ".tiff",
            ".webp",
            ".svg",
        }
        return any(key.lower().endswith(ext) for ext in text_extensions)


    def is_markdown_file(self, key: str) -> bool:
        text_extensions = {
            ".md",
        }
        return any(key.lower().endswith(ext) for ext in text_extensions)



from .live_streaming import  LiveStreamingService
from .tools import register_tools
from ...config import config


def load(cfg: config.Config):
    miku = LiveStreamingService(cfg)
    register_tools(miku)


__all__ = ["load"]



import logging

from qiniu import CdnManager, Auth
from qiniu.http import ResponseInfo
from typing import List, Optional, Dict
from pydantic import BaseModel
from dataclasses import dataclass

from ...consts import consts
from ...config import config

logger = logging.getLogger(consts.LOGGER_NAME)


@dataclass
class PrefetchUrlsResult(BaseModel):
    code: Optional[int] = None
    error: Optional[str] = None
    requestId: Optional[str] = None
    invalidUrls: Optional[List[str]] = None
    quotaDay: Optional[int] = None
    surplusDay: Optional[int] = None


@dataclass
class RefreshResult(BaseModel):
    code: Optional[int] = None
    error: Optional[str] = None
    requestId: Optional[str] = None
    taskIds: Optional[Dict[str, str]] = None
    invalidUrls: Optional[List[str]] = None
    invalidDirs: Optional[List[str]] = None
    urlQuotaDay: Optional[int] = None
    urlSurplusDay: Optional[int] = None
    dirQuotaDay: Optional[int] = None
    dirSurplusDay: Optional[int] = None


def _raise_if_resp_error(resp: ResponseInfo):
    if resp.ok():
        return
    raise RuntimeError(f"qiniu response error: {str(resp)}")


class CDNService:
    def __init__(self, cfg: config.Config):
        auth = Auth(access_key=cfg.access_key, secret_key=cfg.secret_key)
        self._cdn_manager = CdnManager(auth)

    def prefetch_urls(self, urls: List[str] = []) -> PrefetchUrlsResult:
        if not urls:
            raise ValueError("urls is empty")
        info, resp = self._cdn_manager.prefetch_urls(urls)
        _raise_if_resp_error(resp)
        return PrefetchUrlsResult.model_validate(info)

    def refresh(self, urls: List[str] = [], dirs: List[str] = []) -> RefreshResult:
        if not urls and not dirs:
            raise ValueError("urls and dirs cannot be empty")
        info, resp = self._cdn_manager.refresh_urls_and_dirs(urls, dirs)
        _raise_if_resp_error(resp)
        return RefreshResult.model_validate(info)


__all__ = [
    "PrefetchUrlsResult",
    "RefreshResult",
    "CDNService",
]



import logging
import base64

from mcp import types
from mcp.types import ImageContent, TextContent

from .storage import StorageService
from ...consts import consts
from ...tools import tools

logger = logging.getLogger(consts.LOGGER_NAME)

_BUCKET_DESC = "Qiniu Cloud Storage bucket Name"

class _ToolImpl:
    def __init__(self, storage: StorageService):
        self.storage = storage

    @tools.tool_meta(
        types.Tool(
            name="list_buckets",
            description="Return the Bucket you configured based on the conditions.",
            inputSchema={
                "type": "object",
                "properties": {
                    "prefix": {
                        "type": "string",
                        "description": "Bucket prefix. The listed Buckets will be filtered based on this prefix, and only those matching the prefix will be output.",
                    },
                },
                "required": [],
            },
        )
    )
    async def list_buckets(self, **kwargs) -> list[types.TextContent]:
        buckets = await self.storage.list_buckets(**kwargs)
        return [types.TextContent(type="text", text=str(buckets))]

    @tools.tool_meta(
        types.Tool(
            name="list_objects",
            description="List objects in Qiniu Cloud, list a part each time, you can set start_after to continue listing, when the number of listed objects is less than max_keys, it means that all files are listed. start_after can be the key of the last file in the previous listing.",
            inputSchema={
                "type": "object",
                "properties": {
                    "bucket": {
                        "type": "string",
                        "description": _BUCKET_DESC,
                    },
                    "max_keys": {
                        "type": "integer",
                        "description": "Sets the max number of keys returned, default: 20",
                    },
                    "prefix": {
                        "type": "string",
                        "description": "Specify the prefix of the operation response key. Only keys that meet this prefix will be listed.",
                    },
                    "start_after": {
                        "type": "string",
                        "description": "start_after is where you want Qiniu Cloud to start listing from. Qiniu Cloud starts listing after this specified key. start_after can be any key in the bucket.",
                    },
                },
                "required": ["bucket"],
            },
        )
    )
    async def list_objects(self, **kwargs) -> list[types.TextContent]:
        objects = await self.storage.list_objects(**kwargs)
        return [types.TextContent(type="text", text=str(objects))]

    @tools.tool_meta(
        types.Tool(
            name="get_object",
            description="Get an object contents from Qiniu Cloud bucket. In the GetObject request, specify the full key name for the object.",
            inputSchema={
                "type": "object",
                "properties": {
                    "bucket": {
                        "type": "string",
                        "description": _BUCKET_DESC,
                    },
                    "key": {
                        "type": "string",
                        "description": "Key of the object to get.",
                    },
                },
                "required": ["bucket", "key"],
            },
        )
    )
    async def get_object(self, **kwargs) -> list[ImageContent] | list[TextContent]:
        response = await self.storage.get_object(**kwargs)
        file_content = response["Body"]
        content_type = response.get("ContentType", "application/octet-stream")

        # 根据内容类型返回不同的响应
        if content_type.startswith("image/"):
            base64_data = base64.b64encode(file_content).decode("utf-8")
            return [
                types.ImageContent(
                    type="image", data=base64_data, mimeType=content_type
                )
            ]

        if isinstance(file_content, bytes):
            text_content = file_content.decode("utf-8")
        else:
            text_content = str(file_content)
        return [types.TextContent(type="text", text=text_content)]

    @tools.tool_meta(
        types.Tool(
            name="upload_text_data",
            description="Upload text data to Qiniu bucket.",
            inputSchema={
                "type": "object",
                "properties": {
                    "bucket": {
                        "type": "string",
                        "description": _BUCKET_DESC,
                    },
                    "key": {
                        "type": "string",
                        "description": "The key under which a file is saved in Qiniu Cloud Storage serves as the unique identifier for the file within that space, typically using the filename.",
                    },
                    "data": {
                        "type": "string",
                        "description": "The data to upload.",
                    },
                    "overwrite": {
                        "type": "boolean",
                        "description": "Whether to overwrite the existing object if it already exists.",
                    },
                },
                "required": ["bucket", "key", "data"],
            }
        )
    )
    def upload_text_data(self, **kwargs) -> list[types.TextContent]:
        urls = self.storage.upload_text_data(**kwargs)
        return [types.TextContent(type="text", text=str(urls))]

    @tools.tool_meta(
        types.Tool(
            name="upload_local_file",
            description="Upload a local file to Qiniu bucket.",
            inputSchema={
                "type": "object",
                "properties": {
                    "bucket": {
                        "type": "string",
                        "description": _BUCKET_DESC,
                    },
                    "key": {
                        "type": "string",
                        "description": "The key under which a file is saved in Qiniu Cloud Storage serves as the unique identifier for the file within that space, typically using the filename.",
                    },
                    "file_path": {
                        "type": "string",
                        "description": "The file path of file to upload.",
                    },
                    "overwrite": {
                        "type": "boolean",
                        "description": "Whether to overwrite the existing object if it already exists.",
                    },
                },
                "required": ["bucket", "key", "file_path"],
            }
        )
    )
    def upload_local_file(self, **kwargs) -> list[types.TextContent]:
        urls = self.storage.upload_local_file(**kwargs)
        return [types.TextContent(type="text", text=str(urls))]

    @tools.tool_meta(
        types.Tool(
            name="fetch_object",
            description="Fetch a http object to Qiniu bucket.",
            inputSchema={
                "type": "object",
                "properties": {
                    "bucket": {
                        "type": "string",
                        "description": _BUCKET_DESC,
                    },
                    "key": {
                        "type": "string",
                        "description": "The key under which a file is saved in Qiniu Cloud Storage serves as the unique identifier for the file within that space, typically using the filename.",
                    },
                    "url": {
                        "type": "string",
                        "description": "The URL of the object to fetch.",
                    },
                },
                "required": ["bucket", "key", "url"],
            }
        )
    )
    def fetch_object(self, **kwargs) -> list[types.TextContent]:
        urls = self.storage.fetch_object(**kwargs)
        return [types.TextContent(type="text", text=str(urls))]

    @tools.tool_meta(
        types.Tool(
            name="get_object_url",
            description="Get the file download URL, and note that the Bucket where the file is located must be bound to a domain name. If using Qiniu Cloud test domain, HTTPS access will not be available, and users need to make adjustments for this themselves.",
            inputSchema={
                "type": "object",
                "properties": {
                    "bucket": {
                        "type": "string",
                        "description": _BUCKET_DESC,
                    },
                    "key": {
                        "type": "string",
                        "description": "Key of the object to get.",
                    },
                    "disable_ssl": {
                        "type": "boolean",
                        "description": "Whether to disable SSL. By default, it is not disabled (HTTP protocol is used). If disabled, the HTTP protocol will be used.",
                    },
                    "expires": {
                        "type": "integer",
                        "description": "Token expiration time (in seconds) for download links. When the bucket is private, a signed Token is required to access file objects. Public buckets do not require Token signing.",
                    },
                },
                "required": ["bucket", "key"],
            },
        )
    )
    def get_object_url(self, **kwargs) -> list[types.TextContent]:
        urls = self.storage.get_object_url(**kwargs)
        return [types.TextContent(type="text", text=str(urls))]


def register_tools(storage: StorageService):
    tool_impl = _ToolImpl(storage)
    tools.auto_register_tools(
        [
            tool_impl.list_buckets,
            tool_impl.list_objects,
            tool_impl.get_object,
            tool_impl.upload_text_data,
            tool_impl.upload_local_file,
            tool_impl.get_object_url,
        ]
    )



from .cdn import CDNService
from ...consts import consts
from ...tools import tools
import logging
from mcp import types
from typing import Optional, List

logger = logging.getLogger(consts.LOGGER_NAME)


def _build_base_list(
    code: Optional[int],
    error: Optional[str],
    request_id: Optional[str],
) -> List[str]:
    rets = []
    if code:
        rets.append(f"Status Code: {code}")
    if error:
        rets.append(f"Message: {error}")
    if request_id:
        rets.append(f"RequestID: {request_id}")
    return rets


class _ToolImpl:
    def __init__(self, cdn: CDNService):
        self._cdn = cdn

    @tools.tool_meta(
        types.Tool(
            name="cdn_prefetch_urls",
            description="Newly added resources are proactively retrieved by the CDN and stored on its cache nodes in advance. Users simply submit the resource URLs, and the CDN automatically triggers the prefetch process.",
            inputSchema={
                "type": "object",
                "additionalProperties": False,
                "properties": {
                    "urls": {
                        "type": "array",
                        "description": "List of individual URLs to prefetch (max 60 items). Must be full URLs with protocol, e.g. 'http://example.com/file.zip'",
                        "items": {
                            "type": "string",
                            "format": "uri",
                            "pattern": "^https?://",
                            "examples": [
                                "https://cdn.example.com/images/photo.jpg",
                                "http://static.example.com/downloads/app.exe",
                            ],
                        },
                        "maxItems": 60,
                        "minItems": 1,
                    }
                },
                "required": ["urls"],
            },
        )
    )
    def prefetch_urls(self, **kwargs) -> list[types.TextContent]:
        ret = self._cdn.prefetch_urls(**kwargs)

        rets = _build_base_list(ret.code, ret.error, ret.requestId)
        if ret.invalidUrls:
            rets.append(f"Invalid URLs: {ret.invalidUrls}")
        if ret.code // 100 == 2:
            if ret.quotaDay is not None:
                rets.append(f"Today's prefetch quota: {ret.quotaDay}")
            if ret.surplusDay is not None:
                rets.append(f"Today's remaining quota: {ret.surplusDay}")

        return [
            types.TextContent(
                type="text",
                text="\n".join(rets),
            )
        ]

    @tools.tool_meta(
        types.Tool(
            name="cdn_refresh",
            description="This function marks resources cached on CDN nodes as expired. When users access these resources again, the CDN nodes will fetch the latest version from the origin server and store them anew.",
            inputSchema={
                "type": "object",
                "additionalProperties": False,  # 不允许出现未定义的属性
                "properties": {
                    "urls": {
                        "type": "array",
                        "items": {
                            "type": "string",
                            "format": "uri",
                            "pattern": "^https?://",  # 匹配http://或https://开头的URL
                            "examples": ["http://bar.foo.com/index.html"],
                        },
                        "maxItems": 60,
                        "description": "List of exact URLs to refresh (max 60 items). Must be full URLs with protocol, e.g. 'http://example.com/path/page.html'",
                    },
                    "dirs": {
                        "type": "array",
                        "items": {
                            "type": "string",
                            "pattern": "^https?://.*/(\\*|$)",  # 匹配以http://或https://开头的URL，并以/或者以/*结尾的字符串
                            "examples": [
                                "http://bar.foo.com/dir/",
                                "http://bar.foo.com/images/*",
                            ],
                        },
                        "maxItems": 10,
                        "description": "List of directory patterns to refresh (max 10 items). Must end with '/' or '/*' to indicate directory scope",
                    },
                }
            },
        )
    )
    def refresh(self, **kwargs) -> list[types.TextContent]:
        ret = self._cdn.refresh(**kwargs)
        rets = _build_base_list(ret.code, ret.error, ret.requestId)
        if ret.taskIds is not None:
            # 这个可能暂时用不到
            pass
        if ret.invalidUrls:
            rets.append(f"Invalid URLs list: {ret.invalidUrls}")
        if ret.invalidDirs:
            rets.append(f"Invalid dirs: {ret.invalidDirs}")

        if ret.code // 100 == 2:
            if ret.urlQuotaDay is not None:
                rets.append(f"Today's URL refresh quota: {ret.urlQuotaDay}")
            if ret.urlSurplusDay is not None:
                rets.append(f"Today's remaining URL refresh quota: {ret.urlSurplusDay}")
            if ret.dirQuotaDay is not None:
                rets.append(f"Today's directory refresh quota: {ret.dirQuotaDay}")
            if ret.dirSurplusDay is not None:
                rets.append(
                    f"Today's remaining directory refresh quota: {ret.dirSurplusDay}"
                )
        return [
            types.TextContent(
                type="text",
                text="\n".join(rets),
            )
        ]


def register_tools(cdn: CDNService):
    tool_impl = _ToolImpl(cdn)
    tools.auto_register_tools(
        [
            tool_impl.refresh,
            tool_impl.prefetch_urls,
        ]
    )



from .tools import register_tools
from ...config import config
from .cdn import CDNService


def load(cfg: config.Config):
    cdn = CDNService(cfg)
    register_tools(cdn)


__all__ = [
    "load",
]



import aiohttp
import logging

from typing import Dict, Any
from ...config import config
from ...consts import consts

logger = logging.getLogger(consts.LOGGER_NAME)


class LiveStreamingService:
    def __init__(self, cfg: config.Config = None):
        self.config = cfg
        self.api_key = cfg.api_key if cfg else None
        self.endpoint_url = cfg.endpoint_url if cfg else None

    def _get_auth_header(self) -> Dict[str, str]:
        """Generate Bearer token authorization header"""
        if not self.api_key:
            raise ValueError("QINIU_API_KEY is not configured")
        return {
            "Authorization": f"Bearer {self.api_key}"
        }

    def _build_bucket_url(self, bucket: str) -> str:
        """Build S3-style bucket URL"""
        if not self.endpoint_url:
            raise ValueError("QINIU_ENDPOINT_URL is not configured")

        # Remove protocol if present in endpoint_url
        endpoint = self.endpoint_url
        if endpoint.startswith("http://"):
            endpoint = endpoint[7:]
        elif endpoint.startswith("https://"):
            endpoint = endpoint[8:]

        # Build URL in format: https://<bucket>.<endpoint>
        return f"https://{bucket}.{endpoint}"

    def _build_stream_url(self, bucket: str, stream: str) -> str:
        """Build S3-style stream URL"""
        if not self.endpoint_url:
            raise ValueError("QINIU_ENDPOINT_URL is not configured")

        # Remove protocol if present in endpoint_url
        endpoint = self.endpoint_url
        if endpoint.startswith("http://"):
            endpoint = endpoint[7:]
        elif endpoint.startswith("https://"):
            endpoint = endpoint[8:]

        # Build URL in format: https://<bucket>.<endpoint>/<stream>
        return f"https://{bucket}.{endpoint}/{stream}"

    async def create_bucket(self, bucket: str) -> Dict[str, Any]:
        """
        Create a bucket using S3-style API

        Args:
            bucket: The bucket name to create

        Returns:
            Dict containing the response status and message
        """
        url = self._build_bucket_url(bucket)
        headers = self._get_auth_header()

        logger.info(f"Creating bucket: {bucket} at {url}")

        async with aiohttp.ClientSession() as session:
            async with session.put(url, headers=headers) as response:
                status = response.status
                text = await response.text()

                if status == 200 or status == 201:
                    logger.info(f"Successfully created bucket: {bucket}")
                    return {
                        "status": "success",
                        "bucket": bucket,
                        "url": url,
                        "message": f"Bucket '{bucket}' created successfully",
                        "status_code": status
                    }
                else:
                    logger.error(f"Failed to create bucket: {bucket}, status: {status}, response: {text}")
                    return {
                        "status": "error",
                        "bucket": bucket,
                        "url": url,
                        "message": f"Failed to create bucket: {text}",
                        "status_code": status
                    }

    async def create_stream(self, bucket: str, stream: str) -> Dict[str, Any]:
        """
        Create a stream using S3-style API

        Args:
            bucket: The bucket name
            stream: The stream name to create

        Returns:
            Dict containing the response status and message
        """
        url = self._build_stream_url(bucket, stream)
        headers = self._get_auth_header()

        logger.info(f"Creating stream: {stream} in bucket: {bucket} at {url}")

        async with aiohttp.ClientSession() as session:
            async with session.put(url, headers=headers) as response:
                status = response.status
                text = await response.text()

                if status == 200 or status == 201:
                    logger.info(f"Successfully created stream: {stream} in bucket: {bucket}")
                    return {
                        "status": "success",
                        "bucket": bucket,
                        "stream": stream,
                        "url": url,
                        "message": f"Stream '{stream}' created successfully in bucket '{bucket}'",
                        "status_code": status
                    }
                else:
                    logger.error(f"Failed to create stream: {stream}, status: {status}, response: {text}")
                    return {
                        "status": "error",
                        "bucket": bucket,
                        "stream": stream,
                        "url": url,
                        "message": f"Failed to create stream: {text}",
                        "status_code": status
                    }

    async def bind_push_domain(self, bucket: str, domain: str, domain_type: str = "pushRtmp") -> Dict[str, Any]:
        """
        Bind a push domain to the bucket

        Args:
            bucket: The bucket name
            domain: The push domain name
            domain_type: The type of push domain (default: pushRtmp)

        Returns:
            Dict containing the response status and message
        """
        url = f"{self._build_bucket_url(bucket)}/?pushDomain"
        headers = {
            **self._get_auth_header(),
            "Content-Type": "application/json"
        }
        data = {
            "domain": domain,
            "type": domain_type
        }

        logger.info(f"Binding push domain: {domain} (type: {domain_type}) to bucket: {bucket}")

        async with aiohttp.ClientSession() as session:
            async with session.post(url, headers=headers, json=data) as response:
                status = response.status
                text = await response.text()

                if status == 200 or status == 201:
                    logger.info(f"Successfully bound push domain: {domain} to bucket: {bucket}")
                    return {
                        "status": "success",
                        "bucket": bucket,
                        "domain": domain,
                        "type": domain_type,
                        "message": f"Push domain '{domain}' bound successfully to bucket '{bucket}'",
                        "status_code": status
                    }
                else:
                    logger.error(f"Failed to bind push domain: {domain}, status: {status}, response: {text}")
                    return {
                        "status": "error",
                        "bucket": bucket,
                        "domain": domain,
                        "type": domain_type,
                        "message": f"Failed to bind push domain: {text}",
                        "status_code": status
                    }

    async def bind_play_domain(self, bucket: str, domain: str, domain_type: str = "live") -> Dict[str, Any]:
        """
        Bind a playback domain to the bucket

        Args:
            bucket: The bucket name
            domain: The playback domain name
            domain_type: The type of playback domain (default: live)

        Returns:
            Dict containing the response status and message
        """
        url = f"{self._build_bucket_url(bucket)}/?domain"
        headers = {
            **self._get_auth_header(),
            "Content-Type": "application/json"
        }
        data = {
            "domain": domain,
            "type": domain_type
        }

        logger.info(f"Binding playback domain: {domain} (type: {domain_type}) to bucket: {bucket}")

        async with aiohttp.ClientSession() as session:
            async with session.post(url, headers=headers, json=data) as response:
                status = response.status
                text = await response.text()

                if status == 200 or status == 201:
                    logger.info(f"Successfully bound playback domain: {domain} to bucket: {bucket}")
                    return {
                        "status": "success",
                        "bucket": bucket,
                        "domain": domain,
                        "type": domain_type,
                        "message": f"Playback domain '{domain}' bound successfully to bucket '{bucket}'",
                        "status_code": status
                    }
                else:
                    logger.error(f"Failed to bind playback domain: {domain}, status: {status}, response: {text}")
                    return {
                        "status": "error",
                        "bucket": bucket,
                        "domain": domain,
                        "type": domain_type,
                        "message": f"Failed to bind playback domain: {text}",
                        "status_code": status
                    }

    def get_push_urls(self, push_domain: str, bucket: str, stream_name: str) -> Dict[str, Any]:
        """
        Generate push URLs for RTMP and WHIP protocols

        Args:
            push_domain: The push domain
            bucket: The bucket name
            stream_name: The stream name

        Returns:
            Dict containing RTMP and WHIP push URLs
        """
        rtmp_url = f"rtmp://{push_domain}/{bucket}/{stream_name}"
        whip_url = f"https://{push_domain}/{bucket}/{stream_name}.whip"

        logger.info(f"Generated push URLs for stream: {stream_name}")
        return {
            "status": "success",
            "push_domain": push_domain,
            "bucket": bucket,
            "stream_name": stream_name,
            "rtmp_url": rtmp_url,
            "whip_url": whip_url,
            "message": "Push URLs generated successfully"
        }

    def get_play_urls(self, play_domain: str, bucket: str, stream_name: str) -> Dict[str, Any]:
        """
        Generate playback URLs for FLV, M3U8, and WHEP protocols

        Args:
            play_domain: The playback domain
            bucket: The bucket name
            stream_name: The stream name

        Returns:
            Dict containing FLV, M3U8, and WHEP playback URLs
        """
        flv_url = f"https://{play_domain}/{bucket}/{stream_name}.flv"
        m3u8_url = f"https://{play_domain}/{bucket}/{stream_name}.m3u8"
        whep_url = f"https://{play_domain}/{bucket}/{stream_name}.whep"

        logger.info(f"Generated playback URLs for stream: {stream_name}")
        return {
            "status": "success",
            "play_domain": play_domain,
            "bucket": bucket,
            "stream_name": stream_name,
            "flv_url": flv_url,
            "m3u8_url": m3u8_url,
            "whep_url": whep_url,
            "message": "Playback URLs generated successfully"
        }

    async def query_live_traffic_stats(self, begin: str, end: str) -> Dict[str, Any]:
        """
        Query live streaming traffic statistics

        Args:
            begin: Start time in format YYYYMMDDHHMMSS (e.g., 20240101000000)
            end: End time in format YYYYMMDDHHMMSS (e.g., 20240129105148)

        Returns:
            Dict containing traffic statistics
        """
        if not self.endpoint_url:
            raise ValueError("QINIU_ENDPOINT_URL is not configured")

        # Remove protocol and bucket prefix to get base endpoint
        endpoint = self.endpoint_url
        if endpoint.startswith("http://"):
            endpoint = endpoint[7:]
        elif endpoint.startswith("https://"):
            endpoint = endpoint[8:]

        # Remove bucket prefix if present (format: bucket.endpoint)
       # if '.' in endpoint:
       #     parts = endpoint.split('.', 1)
       #     if len(parts) > 1:
       #         endpoint = parts[1]

        url = f"http://{endpoint}/?trafficStats&begin={begin}&end={end}&g=5min&select=flow&flow=downflow"
        headers = {
            **self._get_auth_header(),
            "Content-Type": "application/json"
        }

        logger.info(f"Querying live traffic stats from {begin} to {end}")

        async with aiohttp.ClientSession() as session:
            async with session.get(url, headers=headers) as response:
                status = response.status
                text = await response.text()

                if status == 200:
                    logger.info("Successfully queried live traffic stats")
                    return {
                        "status": "success",
                        "begin": begin,
                        "end": end,
                        "data": text,
                        "message": "Traffic statistics retrieved successfully",
                        "status_code": status
                    }
                else:
                    logger.error(f"Failed to query traffic stats, status: {status}, response: {text}")
                    return {
                        "status": "error",
                        "begin": begin,
                        "end": end,
                        "message": f"Failed to query traffic stats: {text}",
                        "status_code": status
                    }



import time
from urllib import parse

import qiniu

from mcp_server.core.storage.tools import logger

FUNC_POSITION_NONE = "none"
FUNC_POSITION_PREFIX = "prefix"
FUNC_POSITION_SUFFIX = "suffix"


def url_add_processing_func(auth: qiniu.auth, url: str, func: str) -> str:
    func_items = func.split("/")
    func_prefix = func_items[0]

    url_info = parse.urlparse(url)
    new_query = _query_add_processing_func(url_info.query, func, func_prefix)
    new_query = parse.quote(new_query, safe='&=')
    url_info = url_info._replace(query=new_query)
    new_url = parse.urlunparse(url_info)
    new_url = _sign_url(str(new_url), auth)
    return new_url


def _query_add_processing_func(query: str, func: str, func_prefix: str) -> str:
    queries = query.split("&")
    if '' in queries:
        queries.remove('')

    # query 中不包含任何数据
    if len(queries) == 0:
        return func

    # funcs 会放在第一个元素中
    first_query = parse.unquote(queries[0])

    # funcs 不存在
    if len(first_query) == 0:
        queries.insert(0, func)
        return "&".join(queries)

    # first_query 有 = 说明不是 funcs
    if first_query.find("=") >= 0:
        queries.insert(0, func)
        return "&".join(queries)

    queries.remove(queries[0])

    # 未找到当前类别的 func
    if first_query.find(func_prefix) < 0:
        func = first_query + "|" + func
        queries.insert(0, func)
        return "&".join(queries)

    query_funcs = first_query.split("|")
    if '' in query_funcs:
        query_funcs.remove('')

    # 只有一个 func，且和当前 func 相同，拼接其后
    if len(query_funcs) == 1:
        func = first_query + func.removeprefix(func_prefix)
        queries.insert(0, func)
        return "&".join(queries)

    # 多个 func，查看最后一个是否和当前 func 匹配
    last_func = query_funcs[-1]

    # 最后一个不匹配，只用管道符拼接
    if last_func.find(func_prefix) < 0:
        func = first_query + "|" + func
        queries.insert(0, func)
        return "&".join(queries)

    # 最后一个匹配，则直接拼接在后面
    func = first_query + func.removeprefix(func_prefix)
    queries.insert(0, func)
    return "&".join(queries)


def _sign_url(url: str, auth: qiniu.auth) -> str:
    url_info = parse.urlparse(url)
    query = url_info.query
    if ('e=' not in query) or ('token=' not in query):
        return url

    queries = query.split("&")
    if '' in queries:
        queries.remove('')

    # 移除之前的签名信息，但顺序不可变
    expires = 3600
    new_queries = []
    for query_item in queries:
        if query_item.startswith('e='):
            try:
                deadline = int(query_item.removeprefix('e='))
                expires = deadline - int(time.time())
            except Exception as e:
                logger.warning(f"expires parse fail for url:{url} err:{str(e)}")
                expires = 3600
        elif not query_item.startswith('token='):
            new_queries.append(query_item)

    new_query = "&".join(new_queries)
    url_info = url_info._replace(query=new_query)
    new_url = parse.urlunparse(url_info)
    return auth.private_download_url(new_url, expires=expires)



import qiniu

from ...config import config


class MediaProcessingService:
    def __init__(self, cfg: config.Config):
        self.cfg = cfg
        self.auth = qiniu.Auth(cfg.access_key, cfg.secret_key)

    def execute_fop(
        self,
        bucket: str,
        key: str,
        fops: str = None,
        persistent_type: int = None,
        workflow_template_id: str = None,
        pipeline: str = None,
        notify_url: str = None,
    ) -> dict:
        """
        执行持久化处理
        :param bucket:
        :param key:
        :param fops:
        :param persistent_type:
        :param workflow_template_id:
        :param pipeline:
        :param notify_url:
        :return: 返回字典 dict
            获取 persistentId key 为 persistentId
        """

        persistent_fop = qiniu.PersistentFop(
            auth=self.auth, bucket=bucket, pipeline=pipeline, notify_url=notify_url
        )
        result, info = persistent_fop.execute(
            key=key,
            fops=fops,
            persistent_type=persistent_type,
            workflow_template_id=workflow_template_id,
        )
        return result

    def get_fop_status(self, persistent_id: str) -> dict:
        """
        查询 fop 执行状态
        :param persistent_id:
        :return: dict
            持久化处理的状态，详见 https://developer.qiniu.com/dora/1294/persistent-processing-status-query-prefop
        """
        persistent_fop = qiniu.PersistentFop(auth=self.auth, bucket="")
        result, info = persistent_fop.get_status(persistent_id=persistent_id)
        return result



from . import processing
from .tools import register_tools
from ...config import config


def load(cfg: config.Config):
    cli = processing.MediaProcessingService(cfg)
    register_tools(cfg, cli)


__all__ = [
    "load",
]



import logging

from mcp import types

from .live_streaming import LiveStreamingService
from ...consts import consts
from ...tools import tools

logger = logging.getLogger(consts.LOGGER_NAME)

_BUCKET_DESC = "LiveStreaming bucket name"
_STREAM_DESC = "LiveStreaming stream name"


class _ToolImpl:
    def __init__(self, live_streaming: LiveStreamingService):
        self.live_streaming = live_streaming

    @tools.tool_meta(
        types.Tool(
            name="live_streaming_create_bucket",
            description="Create a new bucket in LiveStreaming using S3-style API. The bucket will be created at https://<bucket>.<endpoint_url>",
            inputSchema={
                "type": "object",
                "properties": {
                    "bucket": {
                        "type": "string",
                        "description": _BUCKET_DESC,
                    },
                },
                "required": ["bucket"],
            },
        )
    )
    async def create_bucket(self, **kwargs) -> list[types.TextContent]:
        result = await self.live_streaming.create_bucket(**kwargs)
        return [types.TextContent(type="text", text=str(result))]

    @tools.tool_meta(
        types.Tool(
            name="live_streaming_create_stream",
            description="Create a new stream in LiveStreaming using S3-style API. The stream will be created at https://<bucket>.<endpoint_url>/<stream>",
            inputSchema={
                "type": "object",
                "properties": {
                    "bucket": {
                        "type": "string",
                        "description": _BUCKET_DESC,
                    },
                    "stream": {
                        "type": "string",
                        "description": _STREAM_DESC,
                    },
                },
                "required": ["bucket", "stream"],
            },
        )
    )
    async def create_stream(self, **kwargs) -> list[types.TextContent]:
        result = await self.live_streaming.create_stream(**kwargs)
        return [types.TextContent(type="text", text=str(result))]

    @tools.tool_meta(
        types.Tool(
            name="live_streaming_bind_push_domain",
            description="Bind a push domain to a LiveStreaming bucket for live streaming. This allows you to configure the domain for pushing RTMP/WHIP streams.",
            inputSchema={
                "type": "object",
                "properties": {
                    "bucket": {
                        "type": "string",
                        "description": _BUCKET_DESC,
                    },
                    "domain": {
                        "type": "string",
                        "description": "The push domain name (e.g., mcp-push1.qiniu.com)",
                    },
                    "domain_type": {
                        "type": "string",
                        "description": "The type of push domain (default: pushRtmp)",
                        "default": "pushRtmp",
                    },
                },
                "required": ["bucket", "domain"],
            },
        )
    )
    async def bind_push_domain(self, **kwargs) -> list[types.TextContent]:
        result = await self.live_streaming.bind_push_domain(**kwargs)
        return [types.TextContent(type="text", text=str(result))]

    @tools.tool_meta(
        types.Tool(
            name="live_streaming_bind_play_domain",
            description="Bind a playback domain to a LiveStreaming bucket for live streaming. This allows you to configure the domain for playing back streams via FLV/M3U8/WHEP.",
            inputSchema={
                "type": "object",
                "properties": {
                    "bucket": {
                        "type": "string",
                        "description": _BUCKET_DESC,
                    },
                    "domain": {
                        "type": "string",
                        "description": "The playback domain name (e.g., mcp-play1.qiniu.com)",
                    },
                    "domain_type": {
                        "type": "string",
                        "description": "The type of playback domain (default: live)",
                        "default": "live",
                    },
                },
                "required": ["bucket", "domain"],
            },
        )
    )
    async def bind_play_domain(self, **kwargs) -> list[types.TextContent]:
        result = await self.live_streaming.bind_play_domain(**kwargs)
        return [types.TextContent(type="text", text=str(result))]

    @tools.tool_meta(
        types.Tool(
            name="live_streaming_get_push_urls",
            description="Get push URLs for a stream. Returns RTMP and WHIP push URLs that can be used to push live streams.",
            inputSchema={
                "type": "object",
                "properties": {
                    "push_domain": {
                        "type": "string",
                        "description": "The push domain name",
                    },
                    "bucket": {
                        "type": "string",
                        "description": _BUCKET_DESC,
                    },
                    "stream_name": {
                        "type": "string",
                        "description": "The stream name",
                    },
                },
                "required": ["push_domain", "bucket", "stream_name"],
            },
        )
    )
    async def get_push_urls(self, **kwargs) -> list[types.TextContent]:
        result = self.live_streaming.get_push_urls(**kwargs)
        return [types.TextContent(type="text", text=str(result))]

    @tools.tool_meta(
        types.Tool(
            name="live_streaming_get_play_urls",
            description="Get playback URLs for a stream. Returns FLV, M3U8, and WHEP playback URLs that can be used to play live streams.",
            inputSchema={
                "type": "object",
                "properties": {
                    "play_domain": {
                        "type": "string",
                        "description": "The playback domain name",
                    },
                    "bucket": {
                        "type": "string",
                        "description": _BUCKET_DESC,
                    },
                    "stream_name": {
                        "type": "string",
                        "description": "The stream name",
                    },
                },
                "required": ["play_domain", "bucket", "stream_name"],
            },
        )
    )
    async def get_play_urls(self, **kwargs) -> list[types.TextContent]:
        result = self.live_streaming.get_play_urls(**kwargs)
        return [types.TextContent(type="text", text=str(result))]

    @tools.tool_meta(
        types.Tool(
            name="live_streaming_query_live_traffic_stats",
            description="Query live streaming traffic statistics for a time range. Returns bandwidth and traffic usage data.",
            inputSchema={
                "type": "object",
                "properties": {
                    "begin": {
                        "type": "string",
                        "description": "Start time in format YYYYMMDDHHMMSS (e.g., 20240101000000)",
                    },
                    "end": {
                        "type": "string",
                        "description": "End time in format YYYYMMDDHHMMSS (e.g., 20240129105148)",
                    },
                },
                "required": ["begin", "end"],
            },
        )
    )
    async def query_live_traffic_stats(self, **kwargs) -> list[types.TextContent]:
        result = await self.live_streaming.query_live_traffic_stats(**kwargs)
        return [types.TextContent(type="text", text=str(result))]


def register_tools(live_streaming: LiveStreamingService):
    tool_impl = _ToolImpl(live_streaming)
    tools.auto_register_tools(
        [
            tool_impl.create_bucket,
            tool_impl.create_stream,
            tool_impl.bind_push_domain,
            tool_impl.bind_play_domain,
            tool_impl.get_push_urls,
            tool_impl.get_play_urls,
            tool_impl.query_live_traffic_stats,
        ]
    )



import logging

import qiniu
from mcp import types

from . import utils
from .processing import MediaProcessingService
from ...config import config
from ...consts import consts
from ...tools import tools

logger = logging.getLogger(consts.LOGGER_NAME)

_OBJECT_URL_DESC = "The URL of the image. This can be a URL obtained via the GetObjectURL tool or a URL generated by other Fop tools. Length Constraints: Minimum length of 1."


class _ToolImpl:
    def __init__(self, cfg: config.Config, cli: MediaProcessingService):
        self.auth = qiniu.Auth(cfg.access_key, cfg.secret_key)
        self.client = cli

    @tools.tool_meta(
        types.Tool(
            name="image_scale_by_percent",
            description="""Image scaling tool that resizes images based on a percentage and returns information about the scaled image.
            The information includes the object_url of the scaled image, which users can directly use for HTTP GET requests to retrieve the image content or open in a browser to view the file.
            The image must be stored in a Qiniu Cloud Bucket.
            Supported original image formats: psd, jpeg, png, gif, webp, tiff, bmp, avif, heic. Image width and height cannot exceed 30,000 pixels, and total pixels cannot exceed 150 million.
            """,
            inputSchema={
                "type": "object",
                "properties": {
                    "object_url": {
                        "type": "string", 
                        "description": _OBJECT_URL_DESC
                    },
                    "percent": {
                        "type": "integer",
                        "description": "Scaling percentage, range [1,999]. For example: 90 means the image width and height are reduced to 90% of the original; 200 means the width and height are enlarged to 200% of the original.",
                        "minimum": 1,
                        "maximum": 999
                    },
                },
                "required": ["object_url", "percent"],
            },
        )
    )
    def image_scale_by_percent(
            self, **kwargs
    ) -> list[types.TextContent | types.ImageContent | types.EmbeddedResource]:
        object_url = kwargs.get("object_url", "")
        percent = kwargs.get("percent", "")
        if object_url is None or len(object_url) == 0:
            return [types.TextContent(type="text", text="object_url is required")]

        percent_int = int(percent)
        if percent_int < 1 or percent_int > 999:
            return [
                types.TextContent(type="text", text="percent must be between 1 and 999")
            ]

        func = f"imageMogr2/thumbnail/!{percent}p"
        object_url = utils.url_add_processing_func(auth=self.auth, url=object_url, func=func)
        return [
            types.TextContent(
                type="text",
                text=str(
                    {
                        "object_url": object_url,
                    }
                ),
            )
        ]

    @tools.tool_meta(
        types.Tool(
            name="image_scale_by_size",
            description="""Image scaling tool that resizes images based on a specified width or height and returns information about the scaled image.
            The information includes the object_url of the scaled image, which users can directly use for HTTP GET requests to retrieve the image content or open in a browser to view the file.
            The image must be stored in a Qiniu Cloud Bucket.
            Supported original image formats: psd, jpeg, png, gif, webp, tiff, bmp, avif, heic. Image width and height cannot exceed 30,000 pixels, and total pixels cannot exceed 150 million.
            """,
            inputSchema={
                "type": "object",
                "properties": {
                    "object_url": {
                        "type": "string", 
                        "description": _OBJECT_URL_DESC
                    },
                    "width": {
                        "type": "integer",
                        "description": "Specifies the width for image scaling. The image will be scaled to the specified width, and the height will be adjusted proportionally.",
                        "minimum": 1
                    },
                    "height": {
                        "type": "integer",
                        "description": "Specifies the height for image scaling. The image will be scaled to the specified height, and the width will be adjusted proportionally.",
                        "minimum": 1
                    },
                },
                "required": ["object_url"]
            },
        )
    )
    def image_scale_by_size(
            self, **kwargs
    ) -> list[types.TextContent]:
        object_url = kwargs.get("object_url", "")
        width = kwargs.get("width", "")
        height = kwargs.get("height", "")
        if object_url is None or len(object_url) == 0:
            return [types.TextContent(type="text", text="object_url is required")]

        func = f"{width}x{height}"
        if len(func) == 1:
            return [
                types.TextContent(
                    type="text", text="At least one width or height must be set"
                )
            ]

        func = f"imageMogr2/thumbnail/{func}"
        object_url = utils.url_add_processing_func(auth=self.auth, url=object_url, func=func)
        return [
            types.TextContent(
                type="text",
                text=str(
                    {
                        "object_url": object_url,
                    }
                ),
            )
        ]

    @tools.tool_meta(
        types.Tool(
            name="image_round_corner",
            description="""Image rounded corner tool that processes images based on width, height, and corner radius, returning information about the processed image.
            If only radius_x or radius_y is set, the other parameter will be assigned the same value, meaning horizontal and vertical parameters will be identical.
            The information includes the object_url of the processed image, which users can directly use for HTTP GET requests to retrieve the image content or open in a browser to view the file.
            The image must be stored in a Qiniu Cloud Bucket.
            Supported original image formats: psd, jpeg, png, gif, webp, tiff, bmp, avif, heic. Image width and height cannot exceed 30,000 pixels, and total pixels cannot exceed 150 million.
            Corner radius supports pixels and percentages, but cannot be negative. Pixels are represented by numbers, e.g., 200 means 200px; percentages use !xp, e.g., !25p means 25%.""",
            inputSchema={
                "type": "object",
                "properties": {
                    "object_url": {
                        "type": "string",
                        "description": _OBJECT_URL_DESC
                    },
                    "radius_x": {
                        "type": "string",
                        "description": "Parameter for horizontal corner size. Can use: pixel values (e.g., 200 for 200px) or percentages (e.g., !25p for 25%), all non-negative values."
                    },
                    "radius_y": {
                        "type": "string",
                        "description": "Parameter for vertical corner size. Can use: pixel values (e.g., 200 for 200px) or percentages (e.g., !25p for 25%), all non-negative values."
                    },
                },
                "required": ["object_url"],
            }
        )
    )
    def image_round_corner(self, **kwargs) -> list[types.TextContent]:
        object_url = kwargs.get("object_url", "")
        radius_x = kwargs.get("radius_x", "")
        radius_y = kwargs.get("radius_y", "")
        if object_url is None or len(object_url) == 0:
            return [
                types.TextContent(
                    type="text",
                    text="object_url is required"
                )
            ]

        if (radius_x is None or len(radius_x) == 0) and (radius_y is None or len(radius_y) == 0) is None:
            return [
                types.TextContent(
                    type="text",
                    text="At least one of radius_x or radius_y must be set"
                )
            ]

        if radius_x is None or len(radius_x) == 0:
            radius_x = radius_y
        elif radius_y is None or len(radius_y) == 0:
            radius_y = radius_x

        func = f"roundPic/radiusx/{radius_x}/radiusy/{radius_y}"
        object_url = utils.url_add_processing_func(auth=self.auth, url=object_url, func=func)
        return [
            types.TextContent(
                type="text",
                text=str({
                    "object_url": object_url,
                })
            )
        ]

    @tools.tool_meta(
        types.Tool(
            name="image_info",
            description="Retrieves basic image information, including image format, size, and color model.",
            inputSchema={
                "type": "object",
                "properties": {
                    "object_url": {
                        "type": "string",
                        "description": _OBJECT_URL_DESC
                    },
                },
                "required": ["object_url"],
            },
        )
    )
    def image_info(self, **kwargs) -> list[types.TextContent]:
        object_url = kwargs.get("object_url", "")
        if object_url is None or len(object_url) == 0:
            return [
                types.TextContent(
                    type="text",
                    text="object_url is required"
                )
            ]

        func = "imageInfo"
        object_url = utils.url_add_processing_func(auth=self.auth, url=object_url, func=func)
        return [
            types.TextContent(
                type="text",
                text=str({
                    "object_url": object_url,
                })
            )
        ]

    @tools.tool_meta(
        types.Tool(
            name="get_fop_status",
            description="Retrieves the execution status of a Fop operation.",
            inputSchema={
                "type": "object",
                "properties": {
                    "persistent_id": {
                        "type": "string",
                        "description": "Operation ID returned from executing a Fop operation",
                    },
                },
                "required": ["persistent_id"],
            },
        )
    )
    def get_fop_status(self, **kwargs) -> list[types.TextContent]:
        status = self.client.get_fop_status(**kwargs)
        return [types.TextContent(type="text", text=str(status))]


def register_tools(cfg: config.Config, cli: MediaProcessingService):
    tool_impl = _ToolImpl(cfg, cli)
    tools.auto_register_tools(
        [
            tool_impl.image_scale_by_percent,
            tool_impl.image_scale_by_size,
            tool_impl.image_round_corner,
            tool_impl.image_info,
        ]
    )
